@@ -12,6 +12,7 @@ import zipfile
 import tempfile
 import shutil
 import enum
+import warnings
 
 from ..core import urlopen, get_remote_file
 
@@ -213,6 +214,7 @@ class Request(object):
     def __init__(self, uri, mode, **kwargs):
 
         # General
+        self.raw_uri = uri
         self._uri_type = None
         self._filename = None
         self._extension = None
@@ -244,8 +246,11 @@ class Request(object):
 
         # Set extension
         if self._filename is not None:
-            parts = urlparse(self._filename)
-            ext = Path(parts.path).suffix.lower()
+            if self._uri_type in (URI_FILENAME, URI_ZIPPED):
+                path = self._filename
+            else:
+                path = urlparse(self._filename).path
+            ext = Path(path).suffix.lower()
             self._extension = ext if ext != "" else None
 
     def _parse_uri(self, uri):
@@ -494,11 +499,11 @@ class Request(object):
                     shutil.copyfileobj(self.get_file(), file)
             return self._filename_local
 
-    def finish(self):
-        """finish()
-        For internal use (called when the context of the reader/writer
-        exits). Finishes this request. Close open files and process
-        results.
+    def finish(self) -> None:
+        """Wrap up this request.
+
+        Finishes any pending reads or writes, closes any open files and frees
+        any resources allocated by this request.
         """
 
         if self.mode.io_mode == IOMode.write:
@@ -506,9 +511,9 @@ class Request(object):
             # See if we "own" the data and must put it somewhere
             bytes = None
             if self._filename_local:
-                with open(self._filename_local, "rb") as file:
-                    bytes = file.read()
+                bytes = Path(self._filename_local).read_bytes()
             elif self._file_is_local:
+                self._file_is_local = False
                 bytes = self._file.getvalue()
 
             # Put the data in the right place
@@ -537,7 +542,10 @@ class Request(object):
             try:
                 os.remove(self._filename_local)
             except Exception:  # pragma: no cover
-                pass
+                warnings.warn(
+                    "Failed to delete the temporary file at "
+                    f"`{self._filename_local}`. Please report this issue."
+                )
             self._filename_local = None
 
         # Detach so gc can clean even if a reference of self lingers
@@ -547,6 +555,7 @@ class Request(object):
         """For internal use. In some situations a write action can have
         a result (bytes data). That is obtained with this function.
         """
+        # Is there a reason to disallow reading multiple times?
         self._result, res = None, self._result
         return res
 
