@@ -111,7 +111,7 @@ def test_gif_legacy_pillow(image_files: Path, im_in: str, mode: str):
     """
 
     im_path = image_files / im_in
-    with iio.imopen(im_path, "r", search_legacy_only=True, format="GIF-PIL") as file:
+    with iio.imopen(im_path, "r", legacy_mode=True, plugin="GIF-PIL") as file:
         iio_im = file.read(pilmode=mode)
 
     pil_im = np.asarray(
@@ -399,8 +399,8 @@ def test_legacy_exif_orientation(image_files: Path):
     with iio.imopen(
         image_files / "chelsea_tagged.png",
         "r",
-        search_legacy_only=True,
-        format="PNG-PIL",
+        legacy_mode=True,
+        plugin="PNG-PIL",
     ) as f:
         im_reloaded = np.asarray(f.read()[0])
         im_meta = f.get_meta()
@@ -419,7 +419,7 @@ def test_legacy_exif_orientation(image_files: Path):
 
 def test_incomatible_write_format(tmp_path):
     with pytest.raises(IOError):
-        iio.v3.imopen(tmp_path / "foo.mp3", "w", plugin="pillow")
+        iio.v3.imopen(tmp_path / "foo.mp3", "w", plugin="pillow", legacy_mode=False)
 
 
 def test_write_to_bytes():
@@ -480,3 +480,16 @@ def test_write_to_bytes_jpg():
     bytes_string = iio.v3.imwrite("<bytes>", image, plugin="pillow", format="JPEG")
 
     assert contents == bytes_string
+
+
+def test_write_jpg_to_bytes_io():
+    # this is a regression test
+    # see: https://github.com/imageio/imageio/issues/687
+
+    image = np.zeros((200, 200), dtype=np.uint8)
+    bytes_io = io.BytesIO()
+    iio.v3.imwrite(bytes_io, image, plugin="pillow", format="jpeg", mode="L")
+    bytes_io.seek(0)
+
+    image_from_file = iio.v3.imread(bytes_io, plugin="pillow")
+    assert np.allclose(image_from_file, image)
